/*
# Weather-Reactive LED Closet

This project uses an ESP32 with a light sensor and NeoPixel LED strip to create a weather-reactive lighting system.

## Hardware Requirements
- ESP32 development board
- NeoPixel LED strip
- Light sensor (analog)
- Power supply for LED strip

## Pin Connections
- LED Strip Data: GPIO 5
- Light Sensor: GPIO 34 (Analog input)

## Features
- Detects ambient light to activate the system
- Fetches weather data for Herzliya, Israel
- Changes LED colors based on temperature:
  - Blue: ≤ 18°C (Cold)
  - Blue to Yellow: 18-25°C
  - Yellow to Red: 25-30°C
  - Red: ≥ 30°C (Hot)
- Blinks blue when it's raining

## Video
- https://youtu.be/qRPGj36vkL0?t=104

## Credits
- Theodore_Dai_Maman #211541594
- Omer_Dan #322952466
- Amit_Kaminsky #207487661

## Setup Instructions
1. Install required libraries in Arduino IDE:
   - Adafruit NeoPixel
   - ArduinoJson
   - WiFi
   - HTTPClient

2. Update the following in the code:
   - WiFi credentials (ssid and password)
   - OpenWeatherMap API key
   - Number of LEDs in your strip (NUM_LEDS)
   - Adjust LIGHT_THRESHOLD based on your sensor readings

3. Upload the code to your ESP32

## Notes
- Weather is checked every 5 minutes when light is detected
- System turns off when it's dark
- Temperature thresholds can be adjusted in the code 
*/

#include <WiFi.h>
#include <Adafruit_NeoPixel.h>
#include <Weather.h>
#define BLYNK_TEMPLATE_ID           "Enter Template ID"
#define BLYNK_TEMPLATE_NAME         "Enter Template Name"
#define BLYNK_AUTH_TOKEN            "Enter Auth Token"

#define BLYNK_PRINT Serial

// Add Blynk library
#include <BlynkSimpleEsp32.h>

// WiFi credentials
const char* ssid = "Enter SSID";
const char* password = "Enter Password";

// Weather API settings
const char* apiKey = "Enter API Key";
Weather weather;  // Correct initialization

// Pin definitions
#define LED_PIN 15     // Digital pin for NeoPixel data
#define LIGHT_PIN 34  // Analog pin for light sensor
#define NUM_LEDS 24   // Number of LEDs in your strip

// Temperature thresholds (in Celsius)
#define COLD_TEMP 18
#define WARM_TEMP 25
#define HOT_TEMP 30

// Light sensor threshold
#define LIGHT_THRESHOLD 2000  // Adjust based on your sensor's readings

// Initialize NeoPixel strip
Adafruit_NeoPixel strip = Adafruit_NeoPixel(NUM_LEDS, LED_PIN, NEO_GRB + NEO_KHZ800);

// Weather state variables
float currentTemp = 0;
bool isRaining = false;
unsigned long lastWeatherCheck = 0;
const unsigned long WEATHER_CHECK_INTERVAL = 60000; // Check weather every 1 minute

void setup() {
  Serial.begin(115200);
  Serial.println("\nStarting Weather-Reactive LED Closet");
  
  strip.begin();
  strip.show();
  Serial.println("LED strip initialized");

  // First connect to WiFi
  Serial.print("Connecting to WiFi");
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("\nWiFi Connected!");

  // Then connect to Blynk
  Serial.print("Connecting to Blynk...");
  Blynk.begin(BLYNK_AUTH_TOKEN, ssid, "");
  Serial.println("Connected!");

  // Set location and get initial weather
  weather.setLocation("Enter Location as geographic coordinates e.g:'32.1624,34.8447'"); 
  Serial.println("Getting initial weather data...");
  checkWeather();
}

void loop() {
  Blynk.run();
  
  static bool wasActive = false;  // Track previous light state
  int lightValue = analogRead(LIGHT_PIN);
  bool isActive = (lightValue < LIGHT_THRESHOLD);

  // Only print debug when light state changes
  if (isActive != wasActive) {
    Serial.print(isActive ? "Light detected - system active" : "Dark environment - LEDs off");
    Serial.print(" | Temperature: ");
    Serial.print(currentTemp);
    Serial.println("°C");
    wasActive = isActive;
  }

  if (isActive) {
    if (millis() - lastWeatherCheck >= WEATHER_CHECK_INTERVAL) {
      checkWeather();
      lastWeatherCheck = millis();
    }
    updateLEDs();
  } else {
    strip.clear();
    strip.show();
  }
  
  delay(100);
}

void checkWeather() {
  if (WiFi.status() == WL_CONNECTED) {
    weather.fetchWeatherData();
    
    if (weather.checkSucess()) {
      float oldTemp = currentTemp;
      currentTemp = weather.getTemperature();
      
      if (currentTemp == 0) {
        Serial.println("Warning: Temperature reading is 0, might be an error");
      }
      
      // Send to Blynk if connected
      if (Blynk.connected()) {
        Blynk.virtualWrite(V6, currentTemp);
      }
      
      // Only print if temperature actually changed
      if (oldTemp != currentTemp) {
        Serial.print("Temperature changed from ");
        Serial.print(oldTemp);
        Serial.print("°C to ");
        Serial.print(currentTemp);
        Serial.println("°C");
      }
    }
  }
}

void updateLEDs() {
  // Temperature-based colors only (removed rain effect)
  String colorMode;
  int r, g, b;
  
  if (currentTemp <= COLD_TEMP) {
    colorMode = "COLD (Blue)";
    r = 0; g = 0; b = 255;
  } else if (currentTemp >= HOT_TEMP) {
    colorMode = "HOT (Red)";
    r = 255; g = 0; b = 0;
  } else if (currentTemp <= WARM_TEMP) {
    float ratio = (currentTemp - COLD_TEMP) / (WARM_TEMP - COLD_TEMP);
    colorMode = "COOL-TO-WARM (Blue-Yellow gradient)";
    r = 255 * ratio;
    g = 255 * ratio;
    b = 255 * (1 - ratio);
  } else {
    float ratio = (currentTemp - WARM_TEMP) / (HOT_TEMP - WARM_TEMP);
    colorMode = "WARM-TO-HOT (Yellow-Red gradient)";
    r = 255;
    g = 255 * (1 - ratio);
    b = 0;
  }
  
  Serial.print("Temperature effect - Mode: ");
  Serial.print(colorMode);
  Serial.printf(" | RGB: (%d,%d,%d)\n", r, g, b);
  
  for (int i = 0; i < NUM_LEDS; i++) {
    strip.setPixelColor(i, strip.Color(r, g, b));
  }
  strip.show();
} 